.floating-bar { background-color: @theme_base_color; border-width: 1px; border-style: solid solid none; border-color: @borders; border-radius: 3px 3px 0 0; }

.floating-bar.left { border-left-style: none; border-top-left-radius: 0; }

.floating-bar.right { border-right-style: none; border-top-right-radius: 0; }

.floating-bar:backdrop { background-color: @theme_unfocused_base_color; border-color: @unfocused_borders; }

.incognito-mode headerbar { background-image: -gtk-icontheme("user-not-tracked-symbolic"); background-repeat: no-repeat; background-position: 157px 0; background-size: 64px 64px; color: rgba(0, 0, 0, 0.35); }

.automation-mode headerbar { background-color: #ff9600; color: rgba(0, 0, 0, 0.35); }

.entry_icon_box { margin-left: 2px; margin-right: 5px; }

.entry_icon_box:dir(rtl) { margin-left: 5px; margin-right: 2px; }

.entry_icon { color: mix(@theme_fg_color, @theme_base_color, 0.2); margin: 0 4px; }

.entry_icon:hover { color: @theme_fg_color; }

.entry_icon:active { color: @theme_selected_bg_color; }

.entry_icon:backdrop { color: mix(@theme_unfocused_fg_color, @theme_unfocused_base_color, 0.2); }

.entry_icon.selected { color: @theme_selected_bg_color; }

.entry_icon.starred { color: orange; }

.bookmarks-row button { opacity: 0.25; }

.bookmarks-row:hover button { opacity: 1; }

.bookmark-tag-widget { padding-left: 8px; background-color: rgba(0, 0, 0, 0.1); border-radius: 3px; }

.bookmark-tag-widget label { padding-left: 8px; padding-right: 8px; }

.bookmark-tag-widget image, .bookmark-tag-widget label, .bookmark-tag-widget button:hover image { color: @theme_fg_color; }

.bookmark-tag-widget-selected { background-color: @theme_selected_bg_color; }

.bookmark-tag-widget-selected image, .bookmark-tag-widget-selected label, .bookmark-tag-widget-selected button:hover image { color: @theme_selected_fg_color; }

#fullscreen-popup { padding: 12px; border-radius: 18px; background: rgba(0, 0, 0, 0.65); color: white; }

.url_progress { border: none; background-color: transparent; background-image: none; padding-top: 0px; padding-bottom: 0px; margin-top: 0px; margin-bottom: 6px; }

.url_progress progress { background-color: @theme_selected_bg_color; background-image: none; border: none; min-height: 2px; }

.url_progress trough { border: none; background-color: transparent; background-image: none; }

.tab-attention { background-image: -gtk-gradient(radial, center center, 0, center center, 0.5, to(@theme_selected_bg_color), to(transparent)); background-size: 6px 6px; background-repeat: no-repeat; background-position: center bottom; }

.page-menu-contents { margin-left: 12px; margin-right: 12px; margin-bottom: 12px; }

.search-entry-occurrences-tag { background: none; color: alpha(@theme_fg_color,0.5); box-shadow: none; border: none; margin: 2px; padding: 2px; }

.search-entry-occurrences-tag:backdrop { color: alpha(@theme_unfocused_fg_color,0.5); }

dzlsuggestionpopover > revealer > box > elastic > scrolledwindow > viewport > list > row > grid { margin-left: 3px; }

dzlsuggestionpopover > revealer > box > elastic > scrolledwindow > viewport > list > row > grid .title, dzlsuggestionpopover > revealer > box > elastic > scrolledwindow > viewport > list > row > grid .subtitle { margin-left: 6px; }

.download_progress progress { min-height: 2px; }

.download_progress trough { min-width: 14px; }
